
/**
 * Pagedesigner Media Manager
 *
 * Provides general methods for connecting Drupal media assets with the Grapes asset manager
 */
class PagedesignerMediaManager {

  /**
   * Construct a new Pagedesigner Media Manager.
   * Also, initializes the default media trait.
   *
   * @param {Object} editor
   * @param {Object} jQuery
   * @param {Object} settings
   */
  constructor(editor, jQuery, settings) {
    this.editor = editor;
    this.jQuery = jQuery;

    // Create generic media trait base definition
    this.mediaTrait =
      jQuery.extend(
        {},
        this.editor.TraitManager.defaultTrait,
        {
          afterInit: function () {
            jQuery(this.inputEl).on(
              'click',
              (function (trait) {
                return function (e) {
                  editor.PDMediaManager.loadForm(trait);
                }
              }(this))
            );
          },
          getInputEl: function () {
            if (!this.inputEl) {
              var button = jQuery('<button type="button" style="width:100%"></button>');
              button.text(Drupal.t('Choose') + ' ' + Drupal.t(this.model.get('type')));
              this.inputEl = button.get(0);
            }
            return this.inputEl;
          },
          getRenderValue: function () {
            return this.model.get('value').src;
          },
          setInputValue: function (value) {
            this.model.set('value', value);
          },
          setValueFromAssetManager: function (value) {
            this.model.set('value', value);
          },
          isMultiSelect: false
        }
      );
  }

  /**
   * Create a new trait with the given type/id based on the default media trait.
   *
   * @see https://grapesjs.com/docs/modules/Traits.html#define-new-trait-type
   * @param {String} type The type/id of the new trait.
   * @returns {Object} A trait with the given type/id.
   */
  addTrait(type) {
    this.editor.TraitManager.addType(type,
      this.jQuery.extend({}, this.mediaTrait)
    );
    return this.editor.TraitManager.getType(type);
  }

  /**
   * Load and process the form based on the given trait.
   * Enables searching, paging and filtering according to trait type.
   *
   * @param {Object} trait The current trait.
   */
  loadForm(trait, type) {
    this.editor.AssetManager.add([]);
    this.editor.AssetManager.render();
    editor.runCommand('open-assets', {
      target: trait
    });

    if( !type ){
      type = trait.model.get('type');
    }

    if (this.jQuery('.gjs-am-assets-header .gjs-am-add-asset').length == 0) {
      this.jQuery('.gjs-am-assets-header').empty().append('<div class="gjs-am-add-asset"></div>');
    }
    Drupal.ajax
      (
        {
          url: '/pagedesigner/form/asset/search/' + type
        }
      )
      .execute()
      .done(
        (function (self, type) {
          return function (data) {
            self.processForm(type);
          }
        })(this, type)
      );
  }

  /**
   * Process a loaded form to show only the given type and enable paging.
   *
   * @param {String} type The asset type to show.
   */
  processForm(type) {
    var form = this.jQuery('.gjs-am-assets-header form');
    var showPrev = this.jQuery('<button class="pd-media-prev-button">&lt;&lt;</button>');
    var showNext = this.jQuery('<button class="pd-media-next-button">&gt;&gt;</button>');
    var AssetManager = this.editor.AssetManager;
    this.jQuery('.gjs-am-assets-cont button').each(
      (function (self) {
        return function (i, button) {
          self.jQuery(button).remove();
        }
      }(this))
    );

    var pager = this.jQuery('<input type="hidden" name="page" value="0"></input>');
    form.append(pager);
    form.on('submit', function (e) {
      e.preventDefault();
      var data = form.serialize();
      var url = Drupal.restconsumer.addFormat(form.attr('action')) + '&' + data;
      Drupal.restconsumer.get(url, true).done(function (response) {
        var assets = AssetManager.getAll();
        for (var x in assets) {
          AssetManager.getAll().remove(assets[x]);
        }
        AssetManager.add(response);
        AssetManager.render(AssetManager.getAll().filter(
          function (asset) {
            if (type == '*') {
              return true;
            }
            return asset.get('type') == type
          }
        ));
      });
    });

    if( this.jQuery('.gjs-am-assets-cont').find('.pagination-holder').length == 0 ){
      this.jQuery('.gjs-am-assets-cont').append('<div class="pagination-holder"></div>');
    }else{
      this.jQuery('.gjs-am-assets-cont').find('.pagination-holder').html('');
    }

    this.jQuery('.gjs-am-assets-cont').find('.pagination-holder').append(showPrev);
    showPrev.on('click', function () {
      if (pager.val() > 0) {
        pager.val(parseInt(pager.val()) - 1);
        form.find('.form-actions .form-submit').click();
      }
    });

    this.jQuery('.gjs-am-assets-cont').find('.pagination-holder').append(showNext);
    showNext.on('click', function () {
      if (AssetManager.getAll().length == 20) {
        pager.val(parseInt(pager.val()) + 1);
        form.find('.form-actions .form-submit').click();
      }
    });
    form.find('.form-actions .form-submit').click();
  }
}

/**
 * Initializes the Pagedesigner Media Manager and add it to the Grapes editor object.
 */
(function (jQuery, Drupal) {
  Drupal.behaviors.pagedesigner_media_manager = {
    attach: function (context, settings) {
      jQuery(document).on('pagedesigner-init-traits', function (e, editor) {
        editor.PDMediaManager = new PagedesignerMediaManager(editor, jQuery);
      });
    }
  };
})(jQuery, Drupal);
